<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00004108RenovacaoContratoFactory {
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Retorna todas as organizações que utilizam o SM
    **/
    public function getOrganizacoesSM() {
        $sql = "SELECT NRORG_ID
                FROM OM_PRODUTO_ORG
                WHERE NRPRODUTO_ID = 290";
        
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    /**
     * Retorna os contratos com renovação automática que venceram no dia anterior a data atual
    **/
    public function getContratosVencidosComRenovacaoAutomatica($nrorg) {
        $sql = "SELECT CONT.NRSEQCONTRATO,
                       CONT.NRCONTRATOFCT,
                       NVL(CONT.NRTIPOORCAMENTO, REFE.NRTIPOORCAMENTO) AS NRTIPOORCAMENTO,
                       CONT.IDREAJUSTEAUTOMATICO,
                       CONT.NRDIASPRORGCONTRENOV,
                       CONT.CDOPERULTATU,
                       TO_CHAR(NVL(CONT.DTINIVIGENCONT, CONT.DTINICIOCONTRATO), 'DD/MM/YYYY') AS DTINIVIGENCONT,
                       TO_CHAR(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO), 'DD/MM/YYYY') AS DTFINVIGENCONT,
                       (SELECT SUM(VRTOTITEMCOMDESC)
                        FROM FCT_NEGOCIACAOCONTRATO
                        WHERE NRSEQCONTRATO = CONT.NRSEQCONTRATO
                          AND NRORG         = CONT.NRORG) AS VRCONTRATO
                  FROM FCT_CONTRATO CONT
                  LEFT JOIN FCT_REFEORCACONT REFE
                    ON CONT.NRSEQCONTRATO = REFE.NRSEQCONTRATO
                   AND CONT.NRORG         = REFE.NRORG
                 WHERE CONT.IDRENOVACAOAUTOMATICA = 'S'
                   AND CONT.IDSITUACAOCONTRATO = 'A'
                   AND CONT.IDATIVO = 'S'
                   AND CONT.NRSEQCONTRATO IN (SELECT NRSEQCONTRATO
                                                FROM FCT_NEGOCIACAOCONTRATO
                                               WHERE IDTPFATURA IN ('A', 'X'))
                   AND TRUNC(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO)) <= TRUNC(SYSDATE - 1)
                   AND CONT.NRORG                 = $nrorg";
        
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    /**
     * Atualiza o periodo de vigencia do contrato
    **/
    public function alterarPeriodoVigenciaContrato($params) {
        $sql = "UPDATE FCT_CONTRATO
                SET DTINIVIGENCONT        = TRUNC(TO_DATE(:DTINIVIGENCONT, 'DD/MM/YYYY')),
                    DTFINVIGENCONT        = TRUNC(TO_DATE(:DTINIVIGENCONT, 'DD/MM/YYYY') + (:NRDIASPRORGCONT - 1)),
                    NRDIASPRORGCONT       = :NRDIASPRORGCONT,
                    DTRENOVACAO           = SYSDATE,
                    IDSITUACAOCONTRATO    = 'A'
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRORG       = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * busca as informações de geração de relatório de renovação na tabela de parametros geral
    **/
    public function getParametrosGeracaoRelatorio($nrorg) {
        $sql = "SELECT DSEMAILALERTRENOVCONT,
                       IDPERIODALERTRENOVCONT
                FROM FCT_PARAMETROGERAL
                WHERE NRORG = $nrorg";
        
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    /**
     * insere na tabela temporaria os logs e contratos que deram error
    **/
    public function inserirLogRenovacaoContrato($params) {
        $sql = "INSERT INTO FCT_LOGRENOVACAOCONT (
                    NRSEQCONTRATO,  MSGERROR,       DTRENOVACAO, 
                    DTFINVIGENCONT, DTINIVIGENCONT, VRCONTRATO
                ) VALUES (
                    :NRSEQCONTRATO,  :ERROR,          TRUNC(SYSDATE), 
                    :DTFINVIGENCONT, :DTINIVIGENCONT, :VRCONTRATO
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * deleta da tabela temporaria os logs e contratos que deram error
    **/
    public function deletarLogErroRenovacao(array $params) {
        $sql = "DELETE FROM FCT_LOGRENOVACAOCONT
                 WHERE NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getDatasAnterior($params){
      $sql="SELECT NVL(C.DTFINVIGENCONT, C.DTFIMCONTRATO) AS DTFINVIGENCONT,
                   NVL(C.DTINIVIGENCONT, C.DTINICIOCONTRATO) AS DTINIVIGENCONT
              FROM FCT_CONTRATO C
             WHERE C.NRSEQCONTRATO = :NRSEQCONTRATO"; 
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      return $std->fetchAll();
    }
    
    /**
     * busca os parametros de reajuste dos contratos
    **/
    public function getParametrosReajusteContrato(array $params) {
        $sql = "SELECT NRSEQCONTRATO, 
                       NRCONTRATOFCT, 
                       NRPARCNEGOCIO, 
                       NMPRINCIPALPARC, 
                       CDTPINDICE, 
                       IDPERVALOR, 
                       CDINDICE, 
                       NMINDICE, 
                       NRPERCENTINDICE, 
                       VRINDICE, 
                       NRORG, 
                       DTINCLUSAO 
                FROM TABLE(FCT_BUSCA_INDICEREAJUSTE(:NRSEQCONTRATO, :NRORG))";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * busca os contratos que vencem na data atual que não possuem
     * renovação automática
    **/
    public function getContratosSemRenovacaoAutomatica(array $params) {
        $sql = "SELECT COUNT(CONT.NRSEQCONTRATO) AS QTD
                  FROM FCT_CONTRATO CONT
                 WHERE NVL(CONT.IDRENOVACAOAUTOMATICA, 'N') = 'N'
                   AND CONT.IDSITUACAOCONTRATO = 'A'
                   AND CONT.IDATIVO = 'S'
                   AND CONT.NRSEQCONTRATO IN (SELECT NRSEQCONTRATO
                                                FROM FCT_NEGOCIACAOCONTRATO
                                               WHERE IDTPFATURA IN ('A', 'X'))
                   AND (:PERIODO = 'D' AND TRUNC(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO)) <= TRUNC(SYSDATE)
                    OR :PERIODO = 'S' AND TRUNC(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO)) <= TRUNC(SYSDATE + 7)
                    OR :PERIODO = 'M' AND TRUNC(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO)) <= TRUNC(LAST_DAY(SYSDATE)))
                   AND CONT.NRORG = :NRORG";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function salvarProrrogContrato(array $params) {
        $sql = "INSERT INTO FCT_PRORROGCONTRATO (
                    NRSEQPRORRCONT,     NRSEQCONTRATO,      IDPRORRCONT,
                    DTNVDATAINICIAL,    DTNVDATAFINAL,      DSPRORRCONT,
                    IDDTATIVO,          NRORG,              DTINCLUSAO,
                    NRORGINCLUSAO,      CDOPERINCLUSAO,     IDATIVO,
                    DTANTDTFINAL,       DTANTDTINICIAL
                )
                VALUES (
                    :NRSEQPRORRCONT,     :NRSEQCONTRATO,      :IDPRORRCONT,
                    :DTNVDATAINICIAL,    :DTNVDATAFINAL,      :DSPRORRCONT,
                    :IDDTATIVO,          :NRORG,              SYSDATE,
                    :NRORGINCLUSAO,      :CDOPERINCLUSAO,     :IDATIVO,
                    :DTANTDTFINAL,       :DTANTDTINICIAL
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /**
     * Atualiza os ids de Renovação e Reajuste automático do contrato
    **/
    public function salvarRenovReajAuto($params) {
        $sql = "UPDATE FCT_CONTRATO
                   SET IDRENOVACAOAUTOMATICA = :IDRENOVACAOAUTOMATICA,
                       IDREAJUSTEAUTOMATICO = :IDREAJUSTEAUTOMATICO,
                       DTULTATU = SYSDATE,
                       NRORGULTATU = :NRORG,
                       CDOPERULTATU = :OPERADOR
                 WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Recebe os dados de Indice Financeiro
    **/
    public function getDadosIndFinanc($params) {
        $sql = "SELECT INDREAJ.CDTPINDICE,
                       INDREAJ.IDPERVALOR,
                       INDREAJ.CDINDICE, 
                       INDREAJ.NMINDICE,                    
                       INDREAJ.VRINDICE,
                       INDREAJ.NRPERCENTINDICE,
                       INDREAJ.NRORG
                  FROM (SELECT NRSEQCONTRATO,
                               NRORG,
                               LISTAGG(CDTPINDICE, ', ') 
                                    WITHIN GROUP ( ORDER BY CDTPINDICE) AS CDTPINDICE,
                               LISTAGG(IDPERVALOR, ', ') 
                                    WITHIN GROUP ( ORDER BY IDPERVALOR) AS IDPERVALOR,
                               LISTAGG(CDINDICE, ', ') 
                                    WITHIN GROUP ( ORDER BY CDINDICE) AS CDINDICE,
                               LISTAGG(NMINDICE, ', ') 
                                    WITHIN GROUP ( ORDER BY NMINDICE) AS NMINDICE,
                               SUM((VRINDICE*NRPERCENTINDICE)/100) AS VRINDICE,
                               SUM(NRPERCENTINDICE) AS NRPERCENTINDICE
                         FROM TABLE (FCT_BUSCA_INDICEREAJUSTE(:NRSEQCONTRATO, :NRORG))
                         GROUP BY NRSEQCONTRATO, NRORG) INDREAJ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Atualiza dados relacionados ao reajuste do contrato
    **/
    public function alterarDadosReajusteContrato($params) {
        $sql = "UPDATE FCT_CONTRATO
                SET DTPADREAJCONT = TRUNC(TO_DATE(:DTPADREAJCONT,'DD/MM/YYYY')),
                    NRDIASREAJAUTOMATICO = :NRDIASREAJAUTOMATICO 
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRORG       = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}

?>